//+------------------------------------------------------------------+
//|                                                    FlatTrend.mq4 |
//|                                                       Kirk Sloan |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Kirk Sloan"
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrRoyalBlue
#property indicator_color2  clrRed
#property indicator_color3  clrGold
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_minimum 0
#property indicator_maximum 1
//---- input parameters
extern ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;
extern int             MACD_Fast        = 12;
extern int             MACD_Slow        = 26;
extern int             MACD_MA          = 9;
extern bool            alertsOn         = false;               // Turn alerts on
extern bool            alertsOnCurrent  = true;                // Alerts on currnt (still opened) bar
extern bool            alertsMessage    = true;                // Alerts should show pop-up message
extern bool            alertsSound      = false;               // Alerts should play alert sound
extern bool            alertsPushNotif  = false;               // Alerts should send push notification
extern bool            alertsEmail      = false;               // Alerts should send email
extern bool            arrowsVisible    = false;               // Arrows visible?
extern bool            arrowsOnNewest   = false;               // Arrows drawn on newst bar of higher time frame bar?
extern string          arrowsIdentifier = "ftmacd Arrows1";    // Unique ID for arrows
extern double          arrowsUpperGap   = 0.5;                 // Upper arrow gap
extern double          arrowsLowerGap   = 0.5;                 // Lower arrow gap
extern color           arrowsUpColor    = clrLimeGreen;
extern color           arrowsDnColor    = clrRed;
extern color           arrowsNuColor    = clrGray;
extern int             arrowsUpCode     = 241;
extern int             arrowsDnCode     = 242;
extern int             arrowsNuCode     = 108;

//---- buffers
double Buffer1[];
double Buffer2[];
double Buffer3[];
double trend[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
  IndicatorBuffers(4);
  SetIndexBuffer(0,Buffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
  SetIndexBuffer(1,Buffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
  SetIndexBuffer(2,Buffer3); SetIndexStyle(2,DRAW_HISTOGRAM);
  SetIndexBuffer(3,trend);
  
  indicatorFileName = WindowExpertName();
  returnBars        = TimeFrame==-99;
  TimeFrame         = MathMax(TimeFrame,_Period);
  
  IndicatorShortName(timeFrameToString(TimeFrame)+" FlatTrend w MACD ("+MACD_Fast+","+MACD_Slow+","+MACD_MA+")");  
return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1); 
           if (returnBars) { Buffer1[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (TimeFrame==Period())
   {
     for(i=limit; i>=0; i--)
     {
        double macd   = iMACD(NULL,0,MACD_Fast,MACD_Slow,MACD_MA,PRICE_CLOSE,MODE_MAIN,i); 
        double signal = iMACD(NULL,0,MACD_Fast,MACD_Slow,MACD_MA,PRICE_CLOSE,MODE_SIGNAL,i);
        
        Buffer1[i] = EMPTY_VALUE;
        Buffer2[i] = EMPTY_VALUE;
        Buffer3[i] = EMPTY_VALUE;
        trend[i]   = trend[i+1];
        
        if ( macd>signal && macd>0)                              trend[i]  = 1;   
        if ( macd<signal && macd<0)                              trend[i]  =-1; 
        if ((macd>signal && macd<0) || (macd<signal && macd>0))  trend[i]  = 0;
        if (trend[i] == 1) Buffer1[i] = 1;
        if (trend[i] ==-1) Buffer2[i] = 1;
        if (trend[i] == 0) Buffer3[i] = 1;
        
        //
        //
        //
        //
        //
        
        if (arrowsVisible)
        {
             string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);  
             if (trend[i]!=trend[i+1])
             {
               if (trend[i] == 1)                   drawArrow(i,arrowsUpColor,arrowsUpCode,false);
               if (trend[i] ==-1)                   drawArrow(i,arrowsDnColor,arrowsDnCode, true);
               if (trend[i] == 0 && trend[i+1]== 1) drawArrow(i,arrowsNuColor,arrowsNuCode,false);
               if (trend[i] == 0 && trend[i+1]==-1) drawArrow(i,arrowsNuColor,arrowsNuCode, true);
             }
        }      
     }
     manageAlerts();
   return(0);
   }
  
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         Buffer1[i] = EMPTY_VALUE;
         Buffer2[i] = EMPTY_VALUE;
         Buffer3[i] = EMPTY_VALUE;
         trend[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,MACD_Fast,MACD_Slow,MACD_MA,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsPushNotif,alertsEmail,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsNuColor,arrowsUpCode,arrowsDnCode,arrowsNuCode,3,y); 
            if (trend[i] == 1) Buffer1[i] = 1;
            if (trend[i] ==-1) Buffer2[i] = 1;
            if (trend[i] == 0) Buffer3[i] = 1;
   }
return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      
      //
      //
      //
      //
      //
      
      
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] == 1) doAlert(whichBar,"strong up");
            if (trend[whichBar] ==-1) doAlert(whichBar,"strong down");
            if (trend[whichBar] == 0) doAlert(whichBar,"weak");
         }                
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Flat Trend macd ",doWhat);
             if (alertsMessage)   Alert(message);
             if (alertsEmail)     SendMail(StringConcatenate(Symbol(),"Flat Trend macd"),message);
             if (alertsPushNotif) SendNotification(message);
             if (alertsSound)     PlaySound("alert2.wav");
      }
} 

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //

      datetime time = Time[i]; if (arrowsOnNewest) time += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,time,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

 
   

